#! /usr/bin/env python

#	Copyright 2013 by Reach Technology, Inc.
#	All Rights Reserved

from gimpfu import *
import slcd_common
	

#
# Adds a button to an existing image.  Button image is loaded from the
# specified file, into a new layer.  The SLCD command to draw the button
# is used as the layer name, with the token COORDS used as a placeholder
# for the x/y coordinates.  The button nickname is added as a comment for
# use by the export script.
#

# button types
#	"1		Standard, notify on press"
#	"2		Latching"
#	"20		Latching, initial state 0"
#	"21		Latching, initial state 1"
#	"3		Typematic"
#	"30		Typematic, one beep"
#	"4		Standard, notify on release"
#	"5		Standard, notify on press and release"

# indicies into buttonInfo array
BUTTON_STD_PRESS =		0
BUTTON_LATCHING =		1
BUTTON_LATCHING_0 =		2
BUTTON_LATCHING_1 =		3
BUTTON_TYPEMATIC =		4
BUTTON_TYPEM_1BEEP =	5
BUTTON_STD_RELEASE =	6
BUTTON_STD_PRESS_REL =	7

# indicies into a single entry in the buttonInfo array
BTN_TYPE =	0
BTN_DESC =	1

buttonInfo =	[	# type	desc
					[1,		"	Standard, notify on press"],
					[2,		"	Latching"],
					[20,	"	Latching, initial state 0"],
					[21,	"	Latching, initial state 1"],
					[3,		"	Typematic"],
					[30,	"	Typematic, one beep"],
					[4,		"	Standard, notify on release"],
					[5,		"	Standard, notify on press and release"]
				]


def makeButtonString(inIndex):
	"""Translate a buttonType index into menu entry strings"""
	buttonString = str(buttonInfo[inIndex][BTN_TYPE]) + str(buttonInfo[inIndex][BTN_DESC])
	return buttonString


def Add_Button_Momentary(
			inImage,
			inIndex,
			inType,
			inText,
			inDx,
			inDy,
			inBitmap0Filename,
			inBitmap1Filename,
			inNickName
):
	"""Add a momentary button to an existing screen image."""
	# sanity checks
	if not slcd_common.user_item_is_file(inBitmap0Filename):
		fail("Invalid bitmap file:\n" + inBitmap0Filename)
	if not slcd_common.user_item_is_file(inBitmap1Filename):
		fail("Invalid bitmap file:\n" + inBitmap1Filename)

	try:
		# DEFINE BUTTON - MOMENTARY syntax:
		#		bd <n> x y type "text" dx dy bmp0 bmp1
		#		"COORDS" is used for "x y", and will be replaced with actual x and y values later
		bmp0 = slcd_common.get_bitmap_index(inBitmap0Filename)
		bmp1 = slcd_common.get_bitmap_index(inBitmap1Filename)
		commandLine =	"bd" + \
						" " + str(int(inIndex)) + \
						" " + slcd_common.coords() + \
						" " + str(buttonInfo[inType][BTN_TYPE]) + \
						" \"" + inText + "\"" + \
						" " + str(inDx) + \
						" " + str(inDy) + \
						" " + str(bmp0) + \
						" " + str(bmp1) + \
						" // @" + str(inNickName)	# nickname is added as a comment
		slcd_common.do_bitmap_add(inImage, commandLine, inBitmap0Filename)
		slcd_common.set_user_item(slcd_common.button_bmp0(), inBitmap0Filename)
		slcd_common.set_user_item(slcd_common.button_bmp1(), inBitmap1Filename)
	except Exception as e:
		print e.args[0]
		pdb.gimp_message(e.args[0])
	return


def Add_Button_Latching(
			inImage,
			inIndex,
			inType,
			inUpText,
			inDownText,
			inDx0,
			inDy0,
			inDx1,
			inDy1,
			inBitmap0Filename,
			inBitmap1Filename,
			inNickName
):
	"""Add a latching button to an existing screen image."""
	# sanity checks
	if not slcd_common.user_item_is_file(inBitmap0Filename):
		fail("Invalid bitmap file:\n" + inBitmap0Filename)
	if not slcd_common.user_item_is_file(inBitmap1Filename):
		fail("Invalid bitmap file:\n" + inBitmap1Filename)

	try:
		# DEFINE BUTTON - LATCHING syntax:
		#		bd <n> x y type "text0" "text1" dx0 dy0 dx1 dy1 bmp0 bmp1
		#		"COORDS" is used for "x y", and will be replaced with actual x and y values later
		bmp0 = slcd_common.get_bitmap_index(inBitmap0Filename)
		bmp1 = slcd_common.get_bitmap_index(inBitmap1Filename)
		commandLine = 	"bd" + \
						" " + str(int(inIndex)) + \
						" " + slcd_common.coords() + \
						" " + str(buttonInfo[inType][BTN_TYPE]) + \
						" \"" + inUpText + "\"" + \
						" \"" + inDownText + "\"" + \
						" " + str(inDx0) + \
						" " + str(inDy0) + \
						" " + str(inDx1) + \
						" " + str(inDy1) + \
						" " + str(bmp0) + \
						" " + str(bmp1) + \
						" // @" + str(inNickName)	# nickname is added as a comment
		slcd_common.do_bitmap_add(inImage, commandLine, inBitmap0Filename)
		slcd_common.set_user_item(slcd_common.button_bmp0(), inBitmap0Filename)
		slcd_common.set_user_item(slcd_common.button_bmp1(), inBitmap1Filename)
	except Exception as e:
		print e.args[0]
		pdb.gimp_message(e.args[0])
	return


def Add_Button_CenterText(
			inImage,
			inIndex,
			inType,
			inText0,
			inText1,
			inBitmap0Filename,
			inBitmap1Filename,
			inNickName
):
	"""Add a center-text button to an existing screen image."""
	# sanity checks
	if not slcd_common.user_item_is_file(inBitmap0Filename):
		fail("Invalid bitmap file:\n" + inBitmap0Filename)
	if not slcd_common.user_item_is_file(inBitmap1Filename):
		fail("Invalid bitmap file:\n" + inBitmap1Filename)

	try:
		# DEFINE BUTTON - MOMENTARY syntax:
		#		bdc <n> x y type "text0" ["text1"] bmp0 bmp1
		#		"COORDS" is used for "x y", and will be replaced with actual x and y values later
		if inType in (BUTTON_LATCHING, BUTTON_LATCHING_0, BUTTON_LATCHING_1):
			optText = " \"" + inText1 + "\""
		else:
			optText = ""
		bmp0 = slcd_common.get_bitmap_index(inBitmap0Filename)
		bmp1 = slcd_common.get_bitmap_index(inBitmap1Filename)
		commandLine =	"bdc" + \
						" " + str(int(inIndex)) + \
						" " + slcd_common.coords() + \
						" " + str(buttonInfo[inType][BTN_TYPE]) + \
						" \"" + inText0 + "\"" + \
						optText + \
						" " + str(bmp0) + \
						" " + str(bmp1) + \
						" // @" + str(inNickName)	# nickname is added as a comment
		slcd_common.do_bitmap_add(inImage, commandLine, inBitmap0Filename)
		slcd_common.set_user_item(slcd_common.button_bmp0(), inBitmap0Filename)
		slcd_common.set_user_item(slcd_common.button_bmp1(), inBitmap1Filename)
	except Exception as e:
		print e.args[0]
		pdb.gimp_message(e.args[0])
	return


#
# registrations
#
menu_path = slcd_common.menu_path()+"/_Button"
button_index_base = 0
button_index_max = 127

register(
	"python_fu_Reach_SLCD_Add_Button_Momentary",		# registration name
	"Add a momentary button to an existing screen image."+slcd_common.blurb_common(),	# blurb
	"Add a momentary button to an existing screen image.",	# help
	"Reach Technology, Inc.",							# author
	"Copyright 2013 by Reach Technology, Inc.  All Rights Reserved",	# copyright notice
	"June, 2013",										# date
	"_Momentary...",									# menu item in menu path below
	"*",												# image types
	[													# params
		# type			param name		visible name	default value
		(PF_IMAGE,		"image",		"Image",		None),
#		(PF_INT,		"index",		"Button Index",	slcd_common.get_user_item(slcd_common.button_index(), 0)),
		(PF_SPINNER,	"index",		"Button Index",	button_index_base, (button_index_base, button_index_max, 1)),
		(PF_RADIO,		"type",			"Type",			BUTTON_STD_PRESS,
					(	# desc										value
						(makeButtonString(BUTTON_STD_PRESS),		BUTTON_STD_PRESS),
						(makeButtonString(BUTTON_TYPEMATIC),		BUTTON_TYPEMATIC),
						(makeButtonString(BUTTON_TYPEM_1BEEP),		BUTTON_TYPEM_1BEEP),
						(makeButtonString(BUTTON_STD_RELEASE),		BUTTON_STD_RELEASE),
						(makeButtonString(BUTTON_STD_PRESS_REL),	BUTTON_STD_PRESS_REL),
					)
		),
		(PF_STRING,		"text",			"Text",			"Up"),
		(PF_INT,		"dx",			"dx",			0),
		(PF_INT,		"dy",			"dy",			0),
		(PF_FILENAME,	"bmp0_file",	"bmp0 File",	slcd_common.get_user_closest(slcd_common.button_bmp0())),
		(PF_FILENAME,	"bmp1_file",	"bmp1 File",	slcd_common.get_user_closest(slcd_common.button_bmp1())),
		(PF_STRING,		"nickname",		"NickName",		"NickName"),
	],
	[],													# results
	Add_Button_Momentary,								# function name
	menu = menu_path									# menu path
)


register(
	"python_fu_Reach_SLCD_Add_Button_Latching",			# registration name
	"Add a latching button to an existing screen image."+slcd_common.blurb_common(),	# blurb
	"Add a latching button to an existing screen image.",	# help
	"Reach Technology, Inc.",							# author
	"Copyright 2013 by Reach Technology, Inc.  All Rights Reserved",	# copyright notice
	"June, 2013",										# date
	"_Latching...",										# menu item in menu path below
	"*",												# image types
	[													# params
		# type			param name		visible name	default value
		(PF_IMAGE,		"image",		"Image",		None),
#		(PF_INT,		"index",		"Button Index",	slcd_common.get_user_item(slcd_common.button_index(), 0)),
		(PF_SPINNER,	"index",		"Button Index",	button_index_base, (button_index_base, button_index_max, 1)),
		(PF_RADIO,		"type",			"Type",			BUTTON_LATCHING,
					(	# desc									value
						(makeButtonString(BUTTON_LATCHING),		BUTTON_LATCHING),
						(makeButtonString(BUTTON_LATCHING_0),	BUTTON_LATCHING_0),
						(makeButtonString(BUTTON_LATCHING_1),	BUTTON_LATCHING_1),
					)
		),
		(PF_STRING,		"upText",		"Up Text",		"Up"),
		(PF_STRING,		"dnText",		"Down Text",	"Down"),
		(PF_INT,		"dx0",			"dx0",			0),
		(PF_INT,		"dy0",			"dy0",			0),
		(PF_INT,		"dx1",			"dx1",			0),
		(PF_INT,		"dy1",			"dy1",			0),
		(PF_FILENAME,	"bmp0_file",	"bmp0 File",	slcd_common.get_user_closest(slcd_common.button_bmp0())),
		(PF_FILENAME,	"bmp1_file",	"bmp1 File",	slcd_common.get_user_closest(slcd_common.button_bmp1())),
		(PF_STRING,		"nickname",		"NickName",		"NickName"),
	],
	[],													# results
	Add_Button_Latching,								# function name
	menu = menu_path									# menu path
)


register(
	"python_fu_Reach_SLCD_Add_Button_CenterText",		# registration name
	"Add a center-text button to an existing screen image."+slcd_common.blurb_common(),	# blurb
	"Add a center-text button to an existing screen image.",	# help
	"Reach Technology, Inc.",							# author
	"Copyright 2013 by Reach Technology, Inc.  All Rights Reserved",	# copyright notice
	"June, 2013",										# date
	"_Center Text...",									# menu item in menu path below
	"*",												# image types
	[													# params
		# type			param name		visible name	default value
		(PF_IMAGE,		"image",		"Image",		None),
#		(PF_INT,		"index",		"Button Index",	slcd_common.get_user_item(slcd_common.button_index(), 0)),
		(PF_SPINNER,	"index",		"Button Index",	button_index_base, (button_index_base, button_index_max, 1)),
		(PF_RADIO,		"type",			"Type",			BUTTON_STD_PRESS,
					(	# desc										value
						(makeButtonString(BUTTON_STD_PRESS),		BUTTON_STD_PRESS),
						(makeButtonString(BUTTON_LATCHING),			BUTTON_LATCHING),
						(makeButtonString(BUTTON_LATCHING_0),		BUTTON_LATCHING_0),
						(makeButtonString(BUTTON_LATCHING_1),		BUTTON_LATCHING_1),
						(makeButtonString(BUTTON_TYPEMATIC),		BUTTON_TYPEMATIC),
						(makeButtonString(BUTTON_TYPEM_1BEEP),		BUTTON_TYPEM_1BEEP),
						(makeButtonString(BUTTON_STD_RELEASE),		BUTTON_STD_RELEASE),
						(makeButtonString(BUTTON_STD_PRESS_REL),	BUTTON_STD_PRESS_REL),
					)
		),
		(PF_STRING,		"upText",		"Text0",		"Up"),
		(PF_STRING,		"dnText",		"Text1",		"Down"),
		(PF_FILENAME,	"bmp0_file",	"bmp0 File",	slcd_common.get_user_closest(slcd_common.button_bmp0())),
		(PF_FILENAME,	"bmp1_file",	"bmp1 File",	slcd_common.get_user_closest(slcd_common.button_bmp1())),
		(PF_STRING,		"nickname",		"NickName",		"NickName"),
	],
	[],													# results
	Add_Button_CenterText,								# function name
	menu = menu_path									# menu path
)


main()
