#! /usr/bin/env python

#	Copyright 2013 by Reach Technology, Inc.
#	All Rights Reserved

from gimpfu import *
import slcd_common
import struct

# unspecified meter needle polygon point
UNSPEC_POINT = 99999


#
# Adds a meter to an existing image.  Bitmap for meter is loaded from the
# specified file, into a new layer.  The SLCD command to draw the meter is
# used as the layer name, with the token COORDS used as a placeholder for
# the x/y coordinates.
#

def set_needle_point(inX, inY):
	if inX == UNSPEC_POINT:
		needle_point = ""
	else:
		needle_point = " " + str(inX) + " " + str(inY)
	return needle_point


def Add_Meter(
			inImage,
			inIndex,
			inBmpFile,
			inMinVal, 
			inMaxVal,
			inInitVal,
			inMinAngle,
			inMaxAngle,
			needleColor,
			inX0, inY0,
			inX1, inY1,
			inX2, inY2,
			inX3, inY3,
			inX4, inY4,
			inX5, inY5,
			inX6, inY6,
			inX7, inY7,
			inX8, inY8,
			inX9, inY9,
			inX10, inY10,
			inNickName
):
	"""Add a meter to an existing screen image."""

	# sanity checks
	if not slcd_common.user_item_is_file(inBmpFile):
		fail("Invalid bitmap file:\n" + inBmpFile)
	if inMinVal == inMaxVal:
		fail("Meter minVal and maxVal are the same!")
	if inInitVal < inMinVal or inInitVal > inMaxVal:
		fail("Meter initVal out-of-bounds!")
	if inMinAngle == inMaxAngle:
		fail("Meter minAngle and maxAngle are the same!")

	try:
		# METER DEFINE syntax:
		#		md <idx> <bitmap> <x> <y> <type> <minVal> <maxVal> \
		#			<init_val> <minAngle> <maxAngle> <x0 y0> <x1 y1> . . . [x10 y10]>
		#		"COORDS" is used for "x y", and will be replaced with actual x and y values later
		bmp = slcd_common.get_bitmap_index(inBmpFile)
		meterType = 1		# type is always '1'
		r = int(needleColor[0])
		g = int(needleColor[1])
		b = int(needleColor[2])
		needleColorHex = hex((r << 16) + (g << 8) + b)
		commandLine =	"md" + \
						" " + str(int(inIndex)) + \
						" " + str(bmp) + \
						" " + slcd_common.coords() + \
						" " + str(meterType) + \
						" " + str(inMinVal) + \
						" " + str(inMaxVal) + \
						" " + str(inInitVal) + \
						" " + str(inMinAngle) + \
						" " + str(inMaxAngle) + \
						" " + str(inX0) + " " + str(inY0) + \
						set_needle_point(inX1, inY1) + \
						set_needle_point(inX2, inY2) + \
						set_needle_point(inX3, inY3) + \
						set_needle_point(inX4, inY4) + \
						set_needle_point(inX5, inY5) + \
						set_needle_point(inX6, inY6) + \
						set_needle_point(inX7, inY7) + \
						set_needle_point(inX8, inY8) + \
						set_needle_point(inX9, inY9) + \
						set_needle_point(inX10, inY10) + \
						" // NC:" + needleColorHex[2:] + " @" + str(inNickName)		# needle color and nickname added as a comment

		meterLayer = slcd_common.do_bitmap_add(inImage, commandLine, inBmpFile)
		slcd_common.set_user_item(slcd_common.meter_bmp(), inBmpFile)
		
		# make a temporary layer for the needle so we can rotate it to the
		# specified inMinAngle before placing it on the meter bitmap
		needleLayer = gimp.Layer(inImage, "temp", inImage.width, inImage.height, RGB_IMAGE, 100, NORMAL_MODE)
		needleLayer.add_alpha()	# transparency only works with alpha channels?
		needleLayer.fill(TRANSPARENT_FILL)
		inImage.add_layer(needleLayer, 0)
		
		# outline meter needle on new bitmap
		tmp =	[inX1, inY1, inX2, inY2, inX3, inY3, inX4, inY4, inX5, inY5, \
				inX6, inY6, inX7, inY7, inX8, inY8, inX9, inY9, inX10, inY10]
		needle = []
		pivot =	[inX0, inY0]
		# count valid points
		for i in range(len(tmp)):
			if tmp[i] == UNSPEC_POINT:
				break
			else:
				# needle points are relative to pivot point
				if i & 1 == 0:	# even indicies are 'x' values
					needle.append(inX0 + tmp[i])
				else:			# odd indicies are 'y' values
					needle.append(inY0 + tmp[i])
		# close needle image by duplicating first point at end of list
		needle.append(inX0 + inX1)
		needle.append(inY0 + inY1)
		# set brush to 2-pixel wide circle with hard edges
		brush = pdb.gimp_context_get_brush()
		dup_brush = pdb.gimp_brush_duplicate(brush)
		pdb.gimp_context_set_brush(dup_brush)
		pdb.gimp_brush_set_hardness(dup_brush, 100)
		pdb.gimp_brush_set_shape(dup_brush, BRUSH_GENERATED_CIRCLE)
		pdb.gimp_brush_set_radius(dup_brush, 1)
		# show needle in specified color
		old_fg = pdb.gimp_context_get_foreground()
		pdb.gimp_context_set_foreground(needleColor)
		# show pivot point on meter itself
		pdb.gimp_paintbrush(meterLayer, 0, len(pivot), pivot, PAINT_CONSTANT, 0)
		# show needle on new layer, select needle, then fill it
		pdb.gimp_paintbrush(needleLayer, 0, len(needle), needle, PAINT_CONSTANT, 0)
# couldn't figure the select/fill aspect out, so the needle remains an outline in GIMP.
# the display module renders it correctly though...
#		pdb.gimp_image_select_color(inImage, CHANNEL_OP_ADD, needleLayer, needleColor)
#		pdb.gimp_image_select_item(inImage, CHANNEL_OP_ADD, needleLayer)
#		pdb.gimp_edit_fill(needleLayer, FOREGROUND_FILL)
		# rotate needle - inMinAngle is in degrees, rotate 'angle' parameter is in radians
		pdb.gimp_item_transform_rotate(needleLayer, inMinAngle*0.0175, FALSE, inX0, inY0)
		# merge needleLayer down onto meter background
		pdb.gimp_image_merge_down(inImage, needleLayer, CLIP_TO_BOTTOM_LAYER)
		# restore changed items
		pdb.gimp_context_set_foreground(old_fg)
		pdb.gimp_context_set_brush(brush)
		pdb.gimp_brush_delete(dup_brush)

	except Exception as e:
		print e.args[0]
		pdb.gimp_message(e.args[0])
	return


#
# registrations
#
menu_path = slcd_common.menu_path()+"/_Meter"
meter_index_base = 0
meter_index_max = 7

register(
	"python_fu_Reach_SLCD_Add_Meter",					# registration name
	"Add a meter to an existing screen image."+slcd_common.blurb_common(),	# blurb
	"Add a meter to an existing screen image.",			# help
	"Reach Technology, Inc.",							# author
	"Copyright 2013 by Reach Technology, Inc.  All Rights Reserved",	# copyright notice
	"June, 2013",										# date
	"_Meter...",										# menu item in menu path below
	"*",												# image types
	[													# params
		# type			param name		visible name	default value
		(PF_IMAGE,		"image",		"Image",		None),
#		(PF_INT,		"index",		"Meter Index",	slcd_common.get_user_item(slcd_common.meter_index(), 0)),
		(PF_SPINNER,	"index",		"Meter Index",	meter_index_base, (meter_index_base, meter_index_max, 1)),
		(PF_FILENAME,	"bmp_file",		"bitmap",		slcd_common.get_user_closest(slcd_common.meter_bmp())),
		(PF_INT,		"minVal",		"minVal",		1),
		(PF_INT,		"maxVal",		"maxVal",		1),
		(PF_INT,		"initVal",		"initVal",		1),
		(PF_INT,		"minAngle",		"minAngle",		1),
		(PF_INT,		"maxAngle",		"maxAngle",		1),
		(PF_COLOR,		"needleColor",	"Needle Color",	(255, 0, 0)),
		(PF_INT,		"x0",			"x0 (pivot)",	1),
		(PF_INT,		"y0",			"y0 (pivot)",	1),
		(PF_INT,		"x1",			"x1",			UNSPEC_POINT),		# polygon points relative to pivot
		(PF_INT,		"y1",			"y1",			UNSPEC_POINT),
		(PF_INT,		"x2",			"x2",			UNSPEC_POINT),
		(PF_INT,		"y2",			"y2",			UNSPEC_POINT),
		(PF_INT,		"x3",			"x3",			UNSPEC_POINT),
		(PF_INT,		"y3",			"y3",			UNSPEC_POINT),
		(PF_INT,		"x4",			"x4",			UNSPEC_POINT),
		(PF_INT,		"y4",			"y4",			UNSPEC_POINT),
		(PF_INT,		"x5",			"x5",			UNSPEC_POINT),
		(PF_INT,		"y5",			"y5",			UNSPEC_POINT),
		(PF_INT,		"x6",			"x6",			UNSPEC_POINT),
		(PF_INT,		"y6",			"y6",			UNSPEC_POINT),
		(PF_INT,		"x7",			"x7",			UNSPEC_POINT),
		(PF_INT,		"y7",			"y7",			UNSPEC_POINT),
		(PF_INT,		"x8",			"x8",			UNSPEC_POINT),
		(PF_INT,		"y8",			"y8",			UNSPEC_POINT),
		(PF_INT,		"x9",			"x9",			UNSPEC_POINT),
		(PF_INT,		"y9",			"y9",			UNSPEC_POINT),
		(PF_INT,		"x10",			"x10",			UNSPEC_POINT),
		(PF_INT,		"y10",			"y10",			UNSPEC_POINT),
		(PF_STRING,		"nickname",		"NickName",		"NickName"),
	],
	[],													# results
	Add_Meter,											# function name
	menu = menu_path									# menu path
)


main()
