#! /usr/bin/env python

#	Copyright 2013 by Reach Technology, Inc.
#	All Rights Reserved

from gimpfu import *
import os, sys


#
# Common definitions and functions for Reach SLCD layout utility.
#

# GIMP menu location
REACH_MENU_PATH = "<Image>/File/Create/_ReachTech"

REACH_BLURB_COMMON = "\n" + \
	"\nBitmap file names are assumed to have numbers pre-pended," + "\n" + \
	"as per the \"Bitmap file naming convention\" in the manual."

MAIN_SCREEN = "// MainScreen"
COORDS = "COORDS"
KNOB_OFFSET = "KNOBOFF"
DEFAULT_MACROFILE = "slcd_setup_macro.txt"
NICKNAME_TOKEN = "@"


# item names
REACH_FOLDER = gimp.directory + os.sep + "ReachSLCD"
REACH_BITMAP_FOLDER = "bitmap_folder.txt"
REACH_SETUP_MACRO_SEL = "setup_macro_sel.txt"
REACH_BUTTON_BMP0_SEL = "button_bmp0_sel.txt"
REACH_BUTTON_BMP1_SEL = "button_bmp1_sel.txt"
REACH_SLIDER_BG_SEL = "slider_bg_sel.txt"
REACH_SLIDER_KNOB_SEL = "slider_knob_sel.txt"
REACH_METER_BMP_SEL = "meter_bmp_sel.txt"
REACH_CHART_BMP_SEL = "chart_bmp_sel.txt"
REACH_BITMAP_SEL = "bitmap_sel.txt"
REACH_WINDOWED_BITMAP_SEL = "windowed_bitmap_sel.txt"
REACH_BUTTON_INDEX = "button_index.txt"
REACH_SLIDER_INDEX = "slider_index.txt"
REACH_METER_INDEX = "meter_index.txt"
REACH_CHART_INDEX = "chart_index.txt"
REACH_LEVELBAR_INDEX = "levelbar_index.txt"
REACH_HOTSPOT_INDEX = "hotspot_index.txt"

# helper functions to get item names
def menu_path():
	return REACH_MENU_PATH

def blurb_common():
	return REACH_BLURB_COMMON

def mainscreen():
	return MAIN_SCREEN

def coords():
	return COORDS

def knob_offset():
	return KNOB_OFFSET

def reach_folder():
	return REACH_FOLDER

def bitmap_folder():
	return REACH_BITMAP_FOLDER

def setup_macro():
	return REACH_SETUP_MACRO_SEL

def default_macrofile():
	return DEFAULT_MACROFILE

def nickname_token():
	return NICKNAME_TOKEN

def button_bmp0():
	return REACH_BUTTON_BMP0_SEL

def button_bmp1():
	return REACH_BUTTON_BMP1_SEL

def slider_bg():
	return REACH_SLIDER_BG_SEL

def slider_knob():
	return REACH_SLIDER_KNOB_SEL

def meter_bmp():
	return REACH_METER_BMP_SEL

def chart_bmp():
	return REACH_CHART_BMP_SEL

def bitmap():
	return REACH_BITMAP_SEL

def windowed_bitmap():
	return REACH_WINDOWED_BITMAP_SEL

def button_index():
	return REACH_BUTTON_INDEX

def slider_index():
	return REACH_SLIDER_INDEX

def meter_index():
	return REACH_METER_INDEX

def chart_index():
	return REACH_CHART_INDEX

def levelbar_index():
	return REACH_LEVELBAR_INDEX

def hotspot_index():
	return REACH_HOTSPOT_INDEX


def make_nickname(inNickName):
	""" nickname is added as a comment """
	return "// " + nickname_token() + str(inNickName)


def user_item_is_dir(inUserItem):
	"""Returns True if inUserItem is a folder/directory, else returns False"""
	return os.path.isdir(inUserItem)


def user_item_is_file(inUserItem):
	"""Returns True if inUserItem is a file, else returns False"""
	return os.path.isfile(inUserItem)


def set_user_item(inItem, inItemData):
	"""Store inItemData in file inItem, in main Reach folder"""
	# make sure main folder exists
	if not os.path.exists(reach_folder()):
#		pdb.gimp_message("mkdir " + reach_folder())	# for debug
		os.mkdir(reach_folder(), 0755)
	# then store item in reach_folder
	path = reach_folder() + os.sep + inItem
#	pdb.gimp_message("path: " + path + " - data: " + inItemData)	# for debug
	fd = os.open(path, os.O_RDWR | os.O_CREAT | os.O_TRUNC)
	os.write(fd, inItemData)
	os.close(fd)
	return


def get_user_item(inItem, inDefault=""):
	"""Get contents of file inItem from main Reach folder"""
	path = reach_folder() + os.sep + str(inItem)
# can't use gimp_message here because called during registration phase
#	pdb.gimp_message("get: " + path)			# for debug
	if os.path.exists(path):
		filelen = os.lstat(path).st_size
		fd = os.open(path, os.O_RDONLY)
		itemData = os.read(fd, filelen)
		os.close(fd)
	else:
		# not an error for the file to not exist yet
		itemData = inDefault
	return itemData


def get_user_closest(inItem):
	"""Returns contents of file inItem (if it exists) from main Reach folder,
		or contents of bitmap folder file (if it exists)"""
	itemData = get_user_item(inItem)
#	pdb.gimp_message("closest 1: ->" + itemData + "<-")		# for debug
	if itemData == "":
		itemData = get_user_item(bitmap_folder())
#		pdb.gimp_message("closest 2: ->" + itemData + "<-")	# for debug
	return itemData
	

def get_user_macrofile():
	"""Returns contents of setup_macro (if it exists) from main Reach folder,
		or contents of bitmap folder plus the default macro filename.
		Creates the default file if it doesn't exist to allow the file
		browser to have a default."""
	itemData = get_user_item(setup_macro())
	if itemData == "":
		itemData = get_user_item(bitmap_folder()) + os.sep + default_macrofile()
		os.close(os.open(itemData, os.O_RDWR | os.O_CREAT))
	return itemData
	

def get_bitmap_index(inBitmapFilename):
	"""Returns the bitmap number from the specified filename.
		Assumes Reach-recommended filenames to preserve bitmap order."""
	# isolate the filename
	filename = os.path.basename(inBitmapFilename)
	try:
		bitmapNumber, junk = filename.split("_", 1)
		if not bitmapNumber.isdigit():
			bitmapNumber = "0"
	except:
		bitmapNumber = "0"
#	pdb.gimp_message("index file: " + filename + " - index: " + bitmapNumber)	# for debug
	return bitmapNumber


def do_bitmap_add(inImage, inCommandLine, inBitmapFilename):
	# sanity checks
	if not user_item_is_file(inBitmapFilename):
		raise ValueError("Invalid bitmap file:\n" + inBitmapFilename)

	try:
		# add specified file as new layer to image, using command line as layer name
		layer = pdb.gimp_file_load_layer(inImage, inBitmapFilename)
		layer.name = inCommandLine
		inImage.add_layer(layer, 0)

		# show the new contents
		gimp.displays_flush()

	except Exception as e:
		print e.args[0]
		pdb.gimp_message(e.args[0])
	return layer

