#! /usr/bin/env python

#	Copyright 2013 by Reach Technology, Inc.
#	All Rights Reserved

from gimpfu import *
import slcd_common


#
# Adds a hotspot to an existing image.  Hotspot is drawn as a translucent
# light-gray rectangle, into a new layer.  The SLCD command to draw the
# hotspot is used as the layer name, with the token COORDS used as a
# placeholder for the x/y coordinates.  The hotspot nickname is added as
# a comment for use by the export script.
#

def makeHotspot (inImage, inCommand, inIndex, inWidth, inHeight, inNickName):
	"""Core function for adding a hotspot to an existing image"""
	try:
		# common syntax:
		#		<command> <n> x0 y0 x1 y1
		#		"COORDS" is used for "x0 y0", inWidth and inHeight are used for "x1 y1",
		#		and all will be replaced with actual values later
		commandLine =	inCommand + \
						" " + str(int(inIndex)) + \
						" " + slcd_common.coords() + \
						" " + str(inWidth) + \
						" " + str(inHeight) + \
						" // @" + str(inNickName)	# nickname is added as a comment
#		pdb.gimp_message(commandLine)					# for debug
		layer = gimp.Layer(inImage, commandLine, inWidth, inHeight, RGB_IMAGE, 100, NORMAL_MODE)
		inImage.add_layer(layer, 0)
		old_fg = pdb.gimp_context_get_foreground()
		pdb.gimp_context_set_foreground((192, 192, 192))
		pdb.gimp_edit_fill(layer, FOREGROUND_FILL)
		pdb.gimp_layer_set_opacity(layer, 35)
		gimp.displays_flush()
		pdb.gimp_context_set_foreground(old_fg)
	except Exception as e:
		print e.args[0]
		pdb.gimp_message(e.args[0])
	return


def Add_Hotspot(inImage, inIndex, inWidth, inHeight, inInvisible, inNoBeep, inNickName):
	"""Add a hotspot to an existing screen image."""
	try:
		# DEFINE HOTSPOT syntax:
		#		x <common syntax>
		#			or
		#		xs <common syntax>
		#			or
		#		xsnb <common syntax>
		#		"COORDS" is used for "x0 y0", inWidth and inHeight are used for "x1 y1",
		#		and all will be replaced with actual values later
		if inInvisible:
			visibility = "s"
		else:
			visibility = ""
		if inNoBeep:
			# no-beep forces invisible
			visibility = "s"
			noBeep = "nb"
		else:
			noBeep = ""
		command =	"x" + visibility + noBeep
		makeHotspot(inImage, command, inIndex, inWidth, inHeight, inNickName)
	except Exception as e:
		print e.args[0]
		pdb.gimp_message(e.args[0])
	return


def Add_Typematic_Hotspot(inImage, inIndex, inWidth, inHeight, inInvisible, inNickName):
	"""Add a typematic hotspot to an existing screen image."""
	try:
		# DEFINE HOTSPOT syntax:
		#		xt <common syntax>
		#			or
		#		xst <common syntax>
		#		"COORDS" is used for "x0 y0", inWidth and inHeight are used for "x1 y1",
		#		and all will be replaced with actual values later
		if inInvisible:
			visibility = "s"
		else:
			visibility = ""
		command =	"x" + visibility + "t"
		makeHotspot(inImage, command, inIndex, inWidth, inHeight, inNickName)
	except Exception as e:
		print e.args[0]
		pdb.gimp_message(e.args[0])
	return


def Add_XY_Hotspot(inImage, inIndex, inWidth, inHeight, inNoBeep, inNickName):
	"""Add an XY hotspot to an existing screen image."""
	try:
		# DEFINE HOTSPOT syntax:
		#		xxy <common syntax>
		#			or
		#		xxynb <common syntax>
		#		"COORDS" is used for "x0 y0", inWidth and inHeight are used for "x1 y1",
		#		and all will be replaced with actual values later
		if inNoBeep:
			noBeep = "nb"
		else:
			noBeep = ""
		command =	"xxy" + noBeep
		makeHotspot(inImage, command, inIndex, inWidth, inHeight, inNickName)
	except Exception as e:
		print e.args[0]
		pdb.gimp_message(e.args[0])
	return


#
# registrations
#
menu_path = slcd_common.menu_path()+"/_Hotspot"
blurb_slcd5 = "\n\nNote that the No Beep feature is only available\nwith the SLCD5+ controller."
blurb_index = "\n\nNote that the index value range overlaps the\nslider index value range."
hotspot_index_base = 128
hotspot_index_max = 255
hotspot_default_width = 64
hotspot_default_height = 64

register(
	"python_fu_Reach_Add_Hotspot",						# registration name
	"Add a hotspot to an existing screen image."+blurb_index+blurb_slcd5,	# blurb
	"Add a hotspot to an existing screen image.",		# help
	"Reach Technology, Inc.",							# author
	"Copyright 2013 by Reach Technology, Inc.  All Rights Reserved",	# copyright notice
	"June, 2013",										# date
	"_Hotspot...",										# menu item in menu path below
	"*",												# image types
	[													# params
		# type			param name		visible name				default value
		(PF_IMAGE,		"image",		"Image",					None),
		(PF_SPINNER,	"index",		"Hotspot Index",			hotspot_index_base, (hotspot_index_base, hotspot_index_max, 1)),
		(PF_INT,		"width",		"Width",					hotspot_default_width),
		(PF_INT,		"height",		"Height",					hotspot_default_height),
		(PF_TOGGLE,		"invisible",	"Invisible",				False),
		(PF_TOGGLE,		"noBeep",		"No Beep\n(SLCD5+ only)",	False),
		(PF_STRING,		"nickname",		"NickName",					"NickName"),
	],
	[],													# results
	Add_Hotspot,										# function name
	menu = menu_path									# menu path
)

register(
	"python_fu_Reach_Add_Typematic_Hotspot",			# registration name
	"Add a typematic hotspot to an existing screen image."+blurb_index,	# blurb
	"Add a typematic hotspot to an existing screen image.",				# help
	"Reach Technology, Inc.",							# author
	"Copyright 2013 by Reach Technology, Inc.  All Rights Reserved",	# copyright notice
	"June, 2013",										# date
	"_Typematic Hotspot...",							# menu item in menu path below
	"*",												# image types
	[													# params
		# type			param name		visible name				default value
		(PF_IMAGE,		"image",		"Image",					None),
		(PF_SPINNER,	"index",		"Hotspot Index",			hotspot_index_base, (hotspot_index_base, hotspot_index_max, 1)),
		(PF_INT,		"width",		"Width",					hotspot_default_width),
		(PF_INT,		"height",		"Height",					hotspot_default_height),
		(PF_TOGGLE,		"invisible",	"Invisible",				False),
		(PF_STRING,		"nickname",		"NickName",					"NickName"),
	],
	[],													# results
	Add_Typematic_Hotspot,								# function name
	menu = menu_path									# menu path
)

register(
	"python_fu_Reach_Add_XY_Hotspot",					# registration name
	"Add an XY hotspot to an existing screen image."+blurb_index+blurb_slcd5,	# blurb
	"Add an XY hotspot to an existing screen image.",	# help
	"Reach Technology, Inc.",							# author
	"Copyright 2013 by Reach Technology, Inc.  All Rights Reserved",	# copyright notice
	"June, 2013",										# date
	"_XY Hotspot...",									# menu item in menu path below
	"*",												# image types
	[													# params
		# type			param name		visible name				default value
		(PF_IMAGE,		"image",		"Image",					None),
		(PF_SPINNER,	"index",		"Hotspot Index",			hotspot_index_base, (hotspot_index_base, hotspot_index_max, 1)),
		(PF_INT,		"width",		"Width",					hotspot_default_width),
		(PF_INT,		"height",		"Height",					hotspot_default_height),
		(PF_TOGGLE,		"noBeep",		"No Beep\n(SLCD5+ only)",	False),
		(PF_STRING,		"nickname",		"NickName",					"NickName"),
	],
	[],													# results
	Add_XY_Hotspot,										# function name
	menu = menu_path									# menu path
)

main()
